package hexIT;


import javax.swing.*;
import java.util.*;

// this class represents a configuration of hexagons, a "board".
// specific representations are available... see subclasses HexagonalBoard and DiamondBoard
public abstract class Board implements java.io.Serializable
{
   // tiles/orientation size
   protected int size;

   protected JFrame  frame;
   protected int frameWidth, frameHeight;
   
   public Board()
   {
   }
   
   public Board(int new_size)
   {
      size= new_size;
   }

   public void createFrame(String name)
   {
      frame= new JFrame(name);
      frame.setVisible(false);
   }

   public void setFrameSize(int new_width, int new_height)
   {
      frameWidth= new_width;
      frameHeight= new_height;
      frame.setSize(frameWidth, frameHeight);
   }

   public void show()
   {
      // scale hexagons & images

      frame.setVisible(true);
   }

   public abstract java.util.List getFOV(int hexx, int hexy, int radius);
   public abstract java.util.List getLOS(int hexx1, int hexy1, int hexx2, int hexy2);   

   // ---------------------------------------------------------------------------------------------

   // returns the minimum of 2 integers, but never less than 0
   protected int absmin(int num1, int num2)
   {
      int min= (num1 < num2 ? num1 : num2);
      return (min < 0 ? 0 : min);
   }

   protected int abs(int num)
   {
      return (num < 0 ? (-num) : num);
   }
   
   protected int max(int num1, int num2)
   {
      return (num1 > num2 ? num1 : num2);
   }
   
   protected int sign(int num)
   {
      if (num > 0)
         return 1;
      else if (num < 0)
         return -1;
      else 
         return 0;
   }

   // a distance metric between 2 separate hexagons. this should be the minimum # of hex tiles
   // it takes to walk from one hex to the other... but any distance metric can be used, depending
   // on the need
   public abstract int distance(Hexagon h1, Hexagon h2);

   // returns an iterator over all tiles
   public abstract Iterator tilesIterator();
   
   // finds an objects (not via equals(), via reference!)
   public Hexagon findObject(Drawable obj)
   {
      Iterator i= tilesIterator(); 
      
      while (i.hasNext())
      {
         Hexagon h= (Hexagon)i.next();
         
         if (h.getObj() == obj)
         {
            return h;
         }
      }
      
      return null;
   }
         
         
      
   

}