package hexIT;

import javax.swing.*;
import java.awt.*;
import java.util.*;

public class HexFrame extends JFrame
{
   public static int TYPE_HEXAGONAL=      1;
   public static int TYPE_DIAMOND=        2;

   private int width, height, boardSize;
   private Color drawingColor= Color.gray;

   private HexPanel mainPanel; 

   public HexFrame(String frameName, int new_width, int new_height, int type, int new_boardSize, Color new_bgColor)
   {
      super(frameName); 
      setVisible(false);         

      width= new_width;
      height= new_height;
      boardSize= new_boardSize;

      if (type == HexFrame.TYPE_HEXAGONAL)      
         mainPanel= new HexagonalHexPanel(width, height, boardSize, new_bgColor);
      if (type == HexFrame.TYPE_DIAMOND)      
         mainPanel= new DiamondHexPanel(width, height, boardSize, new_bgColor);
         
      setBackgroundColor(Color.black);         
      setContentPane(mainPanel);         
         
      setSize(width, height);
   }
   
   public void showGrid()
   {
      mainPanel.showGrid(Color.gray);
   }

   public void showGrid(Color drawingColor)
   {
      mainPanel.showGrid(drawingColor);
   }
   
   public void hideGrid()
   {
      mainPanel.hideGrid();
   }
   
   public void setBackgroundColor(Color c)
   {
      setBackground(c);
      mainPanel.setBackground(c);
   }

   public void setDrawingColor(Color new_color)
   {
      drawingColor= new_color;
   }
   
   public void putObject(int x, int y, Drawable obj)
   {
      mainPanel.putObject(x, y, obj);
   }
   
   public void removeObject(int x, int y)
   {   
      mainPanel.removeObject(x, y);
   }

   public HexPanel getHexPanel()
   {
      return mainPanel;
   }
   
   public static void main(String[] args)
   {
      if (args == null || args.length != 2)
      {
         System.out.println("Usage: HexFrame [h|d] <size>");
         System.exit(-1);
      }

      HexFrame hf= null;
      int size= Integer.parseInt(args[1]);

      if (args[0].equals("h"))
         hf= new HexFrame("my frame name", 1024, 768, HexFrame.TYPE_HEXAGONAL, size, Color.black);
      else
      {
         //h= new HexFrame("my frame name", 1024, 768, HexFrame.TYPE_DIAMOND, 4);
         System.out.println("unimplemented");
         System.exit(-1);
      }

      //hf.showGrid();
      Drawable testobj= new DrawableAdapter("gfx/Y8.png");
      hf.putObject(2,2, testobj);
      hf.getHexPanel().setScale(true);
      hf.getHexPanel().setHexOutlineColor(2,2, Color.blue);
      hf.getHexPanel().setHexFillColor(2,2, Color.red);

      //((HexagonalBoard)hf.getHexPanel().getBoard()).getHexagon(3,3).printCoords();
      
      java.util.List l= hf.getHexPanel().getBoard().getFOV(2, 2, 1);
      Iterator i= l.iterator();
      while (i.hasNext())
      {
         Hexagon h= (Hexagon)i.next();
         hf.getHexPanel().setHexOutlineColor(h.getHexX(), h.getHexY(), Color.green);
         hf.getHexPanel().setHexFillColor(h.getHexX(), h.getHexY(), Color.red);
      }

      /*java.util.List l= hf.getHexPanel().getBoard().getLOS(2, 2, 1, 0);
      Iterator i= l.iterator();
      while (i.hasNext())
      {
         Hexagon h= (Hexagon)i.next();
         hf.getHexPanel().setHexOutlineColor(h.getHexX(), h.getHexY(), Color.green);
         hf.getHexPanel().setHexFillColor(h.getHexX(), h.getHexY(), Color.blue);
      }*/
      
      hf.setVisible(true);       
      
      try { Thread.sleep(1000); } catch(Exception e) { }
      ((HexagonalHexPanel)hf.getHexPanel()).moveObject(testobj, HexagonalBoard.DIR_NE); 

      try { Thread.sleep(1000); } catch(Exception e) { }
      ((HexagonalHexPanel)hf.getHexPanel()).moveObject(testobj, HexagonalBoard.DIR_W); 
      
      try { Thread.sleep(1000); } catch(Exception e) { }
      ((HexagonalHexPanel)hf.getHexPanel()).moveObject(testobj, HexagonalBoard.DIR_E);
      
      try { Thread.sleep(1000); } catch(Exception e) { }
      ((HexagonalHexPanel)hf.getHexPanel()).moveObject(testobj, HexagonalBoard.DIR_SW);

      try { Thread.sleep(1000); } catch(Exception e) { }
      ((HexagonalHexPanel)hf.getHexPanel()).moveObject(testobj, HexagonalBoard.DIR_NW);

      try { Thread.sleep(1000); } catch(Exception e) { }
      ((HexagonalHexPanel)hf.getHexPanel()).moveObject(testobj, HexagonalBoard.DIR_SE);
      
   }
         
}



