package hexIT;

import java.awt.*;

public class Hexagon implements java.io.Serializable, Cloneable
{

   // there can be a drawable object associated with this tile. the semantics obviously
   // depends on the application.. this is setup for a most general case. eg, for a game of 'hexwars',
   // this object can be a player's stone.
   private Drawable object;

   // not sure what this means yet.. might need > 1 size parameter
   // represents physical size on screen
   // something to do with the # of pixels, etc
   private int size;

   // pos of top-left corner (pixels)
   private int curX, curY;

   // pos inside representation of the board
   private int hexX, hexY;
   
   private boolean visible;
   private Color borderColor;
   private Color outlineColor;
   private Color fillColor;
   
   // 'coordnums'(indices of these arrays) are ids for vertices of a hexagons: from topmost point
   // going clock clockwise: 0-5
   private int[] pixelX, pixelY;

   // ---------------------------------------------------------------------------------------------
            
   public Hexagon(int new_size, int new_hexX, int new_hexY)
   {
      // by default, no images on the hexagon
      size= new_size;
      hexX= new_hexX;
      hexY= new_hexY;

      borderColor= Color.gray;
      outlineColor= null;
      fillColor= null;
      

      visible= false;
      
      pixelX= new int[6];
      for (int i= 0; i < 6; i++)
         pixelX[i]= -1;
      
      pixelY= new int[6];
      for (int i= 0; i < 6; i++)
         pixelY[i]= -1;
   }

   public int getHexX()
   {
      return hexX;
   }

   public int getHexY()
   {
      return hexY;
   }
   
   public int getPixelX(int coord_num)
   {
      return pixelX[coord_num];
   }
   
   public int getPixelY(int coord_num)
   {
      return pixelY[coord_num];
   }
   
   public void setPixelX(int coord_num, int val)
   {
      pixelX[coord_num]= val;
   }
   
   public void setPixelY(int coord_num, int val)
   {
      pixelY[coord_num]= val;
   }
   
   // coordum 0 must have been placed
   public int getMidPixelX()
   {
      return pixelX[0];
   }
   
   // coordnum 0 must have been places
   public int getMidPixelY()
   {
      return (pixelY[1] + pixelY[2])/2;      
   }
   
   public void putObj(Drawable newObj)
   {
      object= newObj;      
   }
   
   public Drawable getObj()
   {
      return object;
   }

	public boolean isOccupied()
	{
		return (getObj() != null);
	}

   public void scaleObjImage(int new_x, int new_y)
   {
      getObj().setImage(ImageTools.scaleImage(getObj().getImage(), new_x, new_y));
   }
   
   public void removeObj()
   {
      object= null;
   }
   
   public void setBorderColor(Color new_color)
   {
      borderColor= new_color;
   }
   
   public Color getBorderColor()
   {
      return borderColor;
   }
   
   protected void setOutlineColor(Color new_color)
   {
      outlineColor= new_color;
   }
   
   public Color getOutlineColor()
   {
      return outlineColor;
   }

   public boolean hasOutline()
   {
      return (outlineColor != null);
   }
   
   public void removeOutline()
   {
      outlineColor= null;
   }

   public boolean surroundsPixel(int x, int y)
   {
      float slope= ((float)pixelY[5]-(float)pixelY[0])
		              /((float)pixelX[0]-(float)pixelX[5]);
      
      // left half
      if (x >= pixelX[5] && x <= pixelX[0])
      {
         int y_offset= (int)(slope*((float)(x-pixelX[5])));

         if (y >= (pixelY[5] - y_offset) && y <= (pixelY[4] + y_offset))
         {
            return true;
         }
      }
      else if (x > pixelX[0] && x <= pixelX[1])
      {
         int y_offset= (int)(slope*((float)(pixelX[1]-x)));

         if (y >= (pixelY[1] - y_offset) && y <= (pixelY[2] + y_offset))
         {
            return true;
         }
      }
      
      return false;      
   }
   
   public void printCoords()
   {
      System.out.println("("+pixelX[0]+","+pixelY[0]+") ("+pixelX[1]+","+pixelY[1]+") ("+pixelX[2]+","+pixelY[2]+") ("
         +pixelX[3]+","+pixelY[3]+") ("+pixelX[4]+","+pixelY[4]+") ("+pixelX[5]+","+pixelY[5]+")");
   }
   
   protected void setFillColor(Color new_color)
   {
      fillColor= new_color;
   }
   
   public Color getFillColor()
   {
      return fillColor;
   }

   public boolean hasFill()
   {
      return (fillColor != null);
   }
   
   public void removeFill()
   {
      fillColor= null;
   }
   
   public void setVisible(boolean new_visible)
   {
      visible= new_visible;
   }
   
   public boolean isVisible()
   {
      return visible;
   }
   
   protected void place(int new_X, int new_Y)
   {
      curX= new_X;
      curY= new_Y;
   }

   public boolean equals(Object o)
   {
      if (!(o instanceof Hexagon))
         return false;
      
      Hexagon h2= (Hexagon)o;
      
      return (hexX == h2.hexX && hexY == h2.hexY);
   }




}

