package hexIT;

import java.awt.*;
import java.util.*;

public class HexagonalHexPanel extends HexPanel
{
   private double k1, k2, sl;
   private Graphics graphics;
   private LinkedList cached_hexComponents;

   private double sqfk= 0.732050807;       // the "square fit" constant.
                                           // equals to 1/(cos30+sin30). please see notes forexplanation
   // 'coordnums' are ids for vertices of a hexagons: from topmost point going clock clockwise: 0-5

   public HexagonalHexPanel(int new_width, int new_height, int new_size, Color new_bgColor)
   {
      super(new_width, new_height, new_size, new_bgColor);
      board= new HexagonalBoard(new_size);
      bgColor= new_bgColor;
      
      k1= 2+sin30;
      k2= 2*cos30;
      
      // get the best side length for the viewport width. see the notes for the rationale behind these calculuations
      sideLength = (int)(((double)viewportWidth)/(((double)(2*boardSize-1))*k2));
      int curHeight= (int) ( ((double)(2*boardSize))*((double)sideLength)*sin30 +
 ((double)(2*boardSize-1))*((double)sideLength) );
 
      while (curHeight > viewportHeight)
      {
         sideLength -= 1;
         curHeight= (int) ( ((double)(2*boardSize))*((double)sideLength)*sin30 +
 ((double)(2*boardSize-1))*((double)sideLength) );
      }
      
      setViewportHeight(curHeight);

      // we've got an ideal sideLength that will fit the maximum sized hexagonal board in the viewport
      sl= (double)sideLength;
      
      calcPoints();
   }
      
   private void drawHexagon(Hexagon h, Graphics g)
   {
      //graphics= getGraphics();
      int hexx= h.getHexX();
      int hexy= h.getHexY();
            
      //System.out.println("px= " + px + ", py= " + py);
      
      int midx= h.getPixelX(0);
      int midy= h.getPixelY(5) + (int)(0.5*sl);
      
      // draw the actual hexagon
      g.setColor(h.getBorderColor());
      g.drawLine(h.getPixelX(0), h.getPixelY(0), h.getPixelX(1), h.getPixelY(1));            //   \
      g.drawLine(h.getPixelX(1), h.getPixelY(1), h.getPixelX(2), h.getPixelY(2));            //   |
      g.drawLine(h.getPixelX(2), h.getPixelY(2), h.getPixelX(3), h.getPixelY(3));            //   /
      g.drawLine(h.getPixelX(3), h.getPixelY(3), h.getPixelX(4), h.getPixelY(4));            //   _
      g.drawLine(h.getPixelX(4), h.getPixelY(4), h.getPixelX(5), h.getPixelY(5));            //   \
      g.drawLine(h.getPixelX(5), h.getPixelY(5), h.getPixelX(0), h.getPixelY(0));            //   |
      
      // draw the fill 
      if (h.hasFill())
      {
         g.setColor(h.getFillColor());

         // top right         
         int[] triangle1_xpoints= { h.getPixelX(0), h.getPixelX(0), h.getPixelX(1)-1 };
         int[] triangle1_ypoints= { h.getPixelY(0)+1, h.getPixelY(1), h.getPixelY(1) };         
         g.fillPolygon(triangle1_xpoints, triangle1_ypoints, 3);
         // top left         
         int[] triangle2_xpoints= { h.getPixelX(0), h.getPixelX(0), h.getPixelX(5)+1 };
         int[] triangle2_ypoints= { h.getPixelY(0)+1, h.getPixelY(1), h.getPixelY(1) };         
         g.fillPolygon(triangle2_xpoints, triangle2_ypoints, 3);
         // bottom left
         int[] triangle3_xpoints= { h.getPixelX(3), h.getPixelX(3), h.getPixelX(4)+1 };
         int[] triangle3_ypoints= { h.getPixelY(3)-1, h.getPixelY(4), h.getPixelY(4) };
         g.fillPolygon(triangle3_xpoints, triangle3_ypoints, 3);
         // bottom left         
         int[] triangle4_xpoints= { h.getPixelX(3), h.getPixelX(3), h.getPixelX(2)-1 };
         int[] triangle4_ypoints= { h.getPixelY(3)-1, h.getPixelY(2), h.getPixelY(2) };         
         g.fillPolygon(triangle4_xpoints, triangle4_ypoints, 3);
         
         g.fillRect(h.getPixelX(5)+1, h.getPixelY(5), 
            h.getPixelX(2)-h.getPixelX(5)-2,
            h.getPixelY(4)-h.getPixelY(5));
      }
      
      // draw the outline 
      if (h.hasOutline())
      {
         g.setColor(h.getOutlineColor());
         g.drawLine(h.getPixelX(0), h.getPixelY(0)+1, h.getPixelX(1)-1, h.getPixelY(1));           
 //   \
         g.drawLine(h.getPixelX(1)-1, h.getPixelY(1), h.getPixelX(2)-1, h.getPixelY(2));           
 //   |
         g.drawLine(h.getPixelX(2)-1, h.getPixelY(2), h.getPixelX(3), h.getPixelY(3)-1);           
 //   /
         g.drawLine(h.getPixelX(3), h.getPixelY(3)-1, h.getPixelX(4)+1, h.getPixelY(4));           
 //   _
         g.drawLine(h.getPixelX(4)+1, h.getPixelY(4), h.getPixelX(5)+1, h.getPixelY(5));           
 //   \
         g.drawLine(h.getPixelX(5)+1, h.getPixelY(5), h.getPixelX(0), h.getPixelY(0)+1);           
 //   |
      }
      
      // draw an image of the object if there is one
      if (h.getObj() != null)
      {            
         Drawable obj= h.getObj();
         
         if (obj.getImage() != null)
         {
            // draw the image
            Image img= obj.getImage();
            int imgWidth= img.getWidth(this);
            int imgHeight= img.getHeight(this);
            
            g.drawImage(img, midx-imgWidth/2, midy-imgHeight/2, this);
         }
      }      
   }
   
   private int getVertexCoordX(int hexx, int hexy, int coord_num)
   {
      int midx= getMiddlePixelX(hexx, hexy);
      
      switch(coord_num)
      {
         case 0: return midx;
         case 1: return (midx + (int)(sl*cos30));
         case 2: return (midx + (int)(sl*cos30));
         case 3: return midx;
         case 4: return (midx - (int)(sl*cos30));
         case 5: return (midx - (int)(sl*cos30));
         default: return 0;
      }
   }

   private int getVertexCoordY(int hexx, int hexy, int coord_num)
   {
      int midy= getMiddlePixelY(hexx); 

      switch(coord_num)
      {
         case 0: return (midy - (int)(sl*(0.5+sin30)));
         case 1: return (midy - (int)(sl*0.5));
         case 2: return (midy + (int)(sl*0.5));
         case 3: return (midy + (int)(sl*(0.5+sin30)));
         case 4: return (midy + (int)(sl*0.5));
         case 5: return (midy - (int)(sl*0.5));
         default: return 0;
      }
   }
   
   private void calcPoints()
   {
      HexagonalBoard b= (HexagonalBoard)board;
      int sidelen= ((HexagonalBoard)board).getSideLength();
      
      Hexagon h= b.getHexagon(0,0); 
      int startx= getVertexCoordX(0, 0, 5);
      int starty= getVertexCoordY(0, 0, 5);
      
      int starttopx= startx + (int)(sl*cos30); 
      int starttopy= starty - (int)(sl*sin30);

      int startbotx= startx + (int)(sl*cos30);      
      int startboty= starty + (int)sl + (int)(sl*sin30);
            
      // set first row's info
      for (int j= 0; j < sidelen; j++)
      {
         h= b.getHexagon(0, j);         
         
         if (b.isValidCoord(0, j))
         {
            h.setPixelX(5, startx + j*((int)(2.0*sl*cos30)));
            h.setPixelY(5, starty);

            h.setPixelX(1, startx + (j+1)*((int)(2.0*sl*cos30)));
            h.setPixelY(1, starty);
            
            h.setPixelX(4, startx + j*((int)(2.0*sl*cos30)));
            h.setPixelY(4, starty + (int)sl);
         
            h.setPixelX(2, startx + (j+1)*((int)(2.0*sl*cos30)));
            h.setPixelY(2, starty + (int)sl);
            
            h.setPixelX(0, starttopx + j*((int)(2.0*sl*cos30)));
            h.setPixelY(0, starttopy);
            
            h.setPixelX(3, startbotx + j*((int)(2.0*sl*cos30)));
            h.setPixelY(3, startboty);
         }
      }
      
      for (int i= 1; i < sidelen; i++)
         for (int j= 0; j < sidelen; j++)
         {
            h= b.getHexagon(i, j);
            int hexx= h.getHexX();
            int hexy= h.getHexY();            
            
            // check northwest tile 1st
            if (b.isValidCoord(hexx-1, hexy))
            {
               Hexagon ne= b.getHexagon(hexx-1, hexy);
               
               h.setPixelX(0, ne.getPixelX(4));
               h.setPixelY(0, ne.getPixelY(4));
               
               h.setPixelX(1, ne.getPixelX(3));
               h.setPixelY(1, ne.getPixelY(3));
            }
                           
            // check northwest tile 1st
            if (b.isValidCoord(hexx-1, hexy-1))
            {               
               Hexagon nw= b.getHexagon(hexx-1, hexy-1);
               
               h.setPixelX(0, nw.getPixelX(2));
               h.setPixelY(0, nw.getPixelY(2));
               
               h.setPixelX(5, nw.getPixelX(3));
               h.setPixelY(5, nw.getPixelY(3));
            }
            
            if (h.getPixelX(5) == -1)
            {
               h.setPixelX(5, h.getPixelX(0) - (int)(sl*cos30));
               h.setPixelY(5, h.getPixelY(0) + (int)(sl*sin30));
            }
            
            if (h.getPixelX(1) == -1)
            {
               h.setPixelX(1, h.getPixelX(0) + (int)(sl*cos30));
               h.setPixelY(1, h.getPixelY(0) + (int)(sl*sin30));
            }
               
            h.setPixelX(4, h.getPixelX(5));
            h.setPixelY(4, h.getPixelY(5) + (int)sl);
       
            h.setPixelX(2, h.getPixelX(1));
            h.setPixelY(2, h.getPixelY(1) + (int)sl);
      
            h.setPixelX(3, h.getPixelX(4) + (int)(sl*cos30));
            h.setPixelY(3, h.getPixelY(4) + (int)(sl*sin30));
         }
   }            
   
   public void putObject(int x, int y, Drawable obj)
   {
      ((HexagonalBoard)board).getHexagon(x,y).putObj(obj);
      repaint();
   }      
   
   public void removeObject(int x, int y)
   {
      ((HexagonalBoard)board).getHexagon(x,y).removeObj();
      repaint();
   }

   public void moveObject(int x1, int y1, int x2, int y2)
   {
      ((HexagonalBoard)board).moveObject(x1, y1, x2, y2);
      repaint();
   }
   
   public void moveObject(int x1, int y1, int dir)
   {
      ((HexagonalBoard)board).moveObject(x1, y1, dir);
      repaint();
   }

   public void moveObject(Drawable obj, int x2, int y2)
   {
      Hexagon source= board.findObject(obj);
      ((HexagonalBoard)board).moveObject(source.getHexX(), source.getHexY(), x2, y2);     
      
      repaint();
   }

   public void moveObject(Drawable obj, int dir)
   {
      Hexagon source= board.findObject(obj);
      ((HexagonalBoard)board).moveObject(source.getHexX(), source.getHexY(), dir);     

      repaint();
   }
   
   public void paint(Graphics g)
   {
      int x, y;
      Color drawingColor;
      Hexagon h;
                 
      if (g == null)
         return;
      
      g.setColor(bgColor);
      g.fillRect(0, 0, getWidth(), getHeight());
      
      Iterator i= board.tilesIterator();
      //int sidel= ((HexagonalBoard)board).getSideLength();
      
      //for (int i= 0; i < sidel; i++)
      //   for (int j= 0; j < sidel; j++)
      while (i.hasNext())
      {                
         h= (Hexagon)i.next();
         //if (((HexagonalBoard)board).isValidCoord(i,j))
         //{
            //h= ((HexagonalBoard)board).getHexagon(i,j);
            drawHexagon(h, g);
         //}
      }
   }
   
   public void showGrid(Color drawingColor)
   {      
      Iterator i= board.tilesIterator();
      
      while (i.hasNext())
      {
         Hexagon h= (Hexagon)i.next();
         
         h.setVisible(true);
         h.setBorderColor(drawingColor);         
      }
      
      repaint();
   }

   public void hideGrid()
   {
      Iterator i= board.tilesIterator();
      
      while (i.hasNext())
      {
         Hexagon h= (Hexagon)i.next();
         
         h.setVisible(false);
      }
      
      repaint();
   }
   
   public void scaleAllImages()
   {
      Iterator i= board.tilesIterator();

      while (i.hasNext())
      {
         Hexagon h= (Hexagon)i.next();
         
         if (h.getObj() != null)
         {
            if (h.getObj().getImage() != null)
            {
               // see notes
               h.scaleObjImage(((int)(2.0*sqfk*sl*cos30)), ((int)(2.0*(1.0-sqfk)*sl*sin30 + sl)));
            }
         }
      }

      repaint();
   }

   // set the hexagones drawing color to the specified color
   public void setHexOutlineColor(int hexx, int hexy, Color color)
   {
      Hexagon h= ((HexagonalBoard)board).getHexagon(hexx, hexy);
      h.setOutlineColor(color);
      
      repaint();
   }
   
   public void setHexFillColor(int hexx, int hexy, Color color)
   {
      Hexagon h= ((HexagonalBoard)board).getHexagon(hexx, hexy);
      h.setFillColor(color);
      
      repaint();
   }
   
   private int rowWidth(int hexX)
   {
      int curLength= 0;

      if (hexX <= (boardSize-1))
         curLength= (boardSize+hexX);
      else
         curLength= ((2*boardSize-1)-(hexX-(boardSize-1)));

      return curLength;
   }

   private int getMiddlePixelY(int hexX)
   {
      return (int)((sl*(0.5+sin30)) + ((double)hexX)*sl*(1.0+sin30) + (double)topLeftY);
   }

   private int getMiddlePixelX(int hexX, int hexY)
   {
      int curLength= rowWidth(hexX);
      //System.out.println("cl= " + curLength + ", hexx = " + hexX + ", hexy = " +hexY);

      double halfPoint= ((double)viewportWidth)/2.0;
      double offset= (((double)curLength)*sl*cos30);      // short for (curRowWidth/2.0) * (2.0*sl*cos30)
      double x_0= ((double)topLeftX) + halfPoint - offset + sl*cos30;
      
      if (hexX <= (boardSize-1))
      {
         return (int)(x_0 + 2.0*sl*cos30*((double)(hexY)));
      }
      else
      {
         return (int)(x_0 + 2.0*sl*cos30*((double)(hexY-(hexX-(boardSize-1)))));
      }
   }

}

