package server;

import java.io.*;
import java.util.*;
import java.net.*;
import java.awt.*;
import java.awt.event.*;

import hexIT.*;

public class HumanPlayer extends Thread implements MouseListener
{
	private Main main;
	private HexPanel hhp;
	private int color, port;
	private String name;
	private Socket socket;
	private BufferedReader sockIn;
	private PrintStream sockOut;

	private Hexagon selectedHex;
	private boolean connected;
	private boolean selected;
	private boolean myturn;

	public HumanPlayer(Main _main, HexPanel _hhp, String _name, int _port)
	{
		main= _main;
		hhp= _hhp;
		name= _name;
		port= _port;

		selected= false;
		connected= false;
		myturn= false;
	}

	public void run()
	{
		// do something
		// add mouse listeners, etc
		hhp.addMouseListener(this);
		String inputLine;

		try
		{
			while(true)
			{
				if (connected)
				{
					inputLine= sockIn.readLine();
					processMsg(inputLine);
				}
			}
		}
		catch (Exception e)
		{
			e.printStackTrace();
			System.exit(-1);
		}

	}

	private synchronized void processMsg(String msg)
	{
		String[] words= msg.split(" ");
		System.out.println("["+name+"] Processing message: " + msg);

		if (words[0].equals("GAME") && words[1].equals("STARTED;"))
		{
			color= main.convertColor(words[4].substring(0,words[4].length()-1));
			if (color == Main.COLOR_YELLOW)
				myturn= true;	 // yellow starts
		}
		else if (words[0].equals("BLACK") || words[0].equals("YELLOW"))
		{
			if (main.convertColor(words[0]) != color)
			{
				System.out.println("[" + name + "] My turn");
				myturn= true;
			}
		}
	}

	private void connect()
	{
		try
		{
			System.out.println("[" + name + "] Connecting to server...");
			socket= new Socket("localhost", port);
			sockIn= new BufferedReader(new InputStreamReader(socket.getInputStream()));
			sockOut= new PrintStream(socket.getOutputStream());
			connected= true;
		}
		catch(Exception e)
		{
			e.printStackTrace();
			System.exit(-1);
		}
	}

	private void send(String msg)
	{
		sockOut.println(msg);
	}

	private Hexagon hexClicked(int x, int y)
	{
		Iterator i= main.getBoard().tilesIterator();

		while (i.hasNext())
		{
			Hexagon h= (Hexagon)i.next();

			if (h.surroundsPixel(x, y))
				return h;
		}

		return null;
	}

	public synchronized void mousePressed(MouseEvent e)
	{
		//saySomething("Mouse pressed; # of clicks: "
		//+ e.getClickCount(), e);
		if (!connected)
		{
			connect();
			send("START " + name);
			return;
		}

		//System.out.println("Woohoo!");
		if (myturn)
		{
			if (!selected)
			{
				Hexagon h= hexClicked(e.getX(), e.getY());
				if (h != null)
				{
					hhp.setHexOutlineColor(h.getHexX(), h.getHexY(), Color.blue);
					selected= true;
					selectedHex= h;
				}
			}
			else
			{
				Hexagon h= hexClicked(e.getX(), e.getY());

				if (h != null)
				{
					if (h.equals(selectedHex))
					{
						h.removeOutline();
						hhp.repaint();
						selected= false;
					}
					else
					{
						int val= main.getBoard().isStraightPath(selectedHex.getHexX(), selectedHex.getHexY(),
				   	                                        h.getHexX(), h.getHexY());

						if (val != -1)
						{
							String dir= null;

							switch(val)
							{
								case HexagonalBoard.DIR_NW: dir= "NW"; break;
								case HexagonalBoard.DIR_NE: dir= "NE"; break;
								case HexagonalBoard.DIR_W: dir= "W"; break;
								case HexagonalBoard.DIR_E: dir= "E"; break;
								case HexagonalBoard.DIR_SW: dir= "SW"; break;
								case HexagonalBoard.DIR_SE: dir= "SE"; break;
							}

							String move= main.convertColor(color) + " " + selectedHex.getHexX()
								+ " " + selectedHex.getHexY() + " " + dir;

							send(move);

							selectedHex.removeOutline();
							hhp.repaint();
							selected= false;
							myturn= false;
						}
					}
				}
			}
		}
	}

	public void mouseReleased(MouseEvent e) {}
	public void mouseEntered(MouseEvent e) {}
	public void mouseExited(MouseEvent e) {}
	public void mouseClicked(MouseEvent e) {}
}
