package server;

import java.io.*;
import java.util.*;
import java.net.*;
import java.awt.*;

import hexIT.*;

//                0 1 2 3 4 5 6
//               / / / / / / / 7
//          0-  o o o o o o o / 8     y
//         1-  o . . . . . . o / 9
// x      2-  o . . . . . . . o / 10
//       3-  o . . . . . . . . o / 11
//      4-  o . . . . . . . . . o / 12
//     5-  o . . . . . . . . . . o /
//    6-  o . . . . . . . . . . . o
//     7-  o . . . . . . . . x . o    <--- that 'x' is at loc (x,y) = (7,10)
//      8-  o . . . . . . . . . o
//       9-  o . . . . . . . . o
//       10-  o . . . . . . . o
//        11-  o . . . . . . o
//         12-  o o o o o o o


public class Main extends Thread
{
	public static Random RNG= new Random();

	public static final int COLOR_YELLOW= 0;
	public static final int COLOR_BLACK= 1;

	public static final int DIR_NW= 0;
	public static final int DIR_NE= 1;
	public static final int DIR_W= 2;
	public static final int DIR_E= 3;
	public static final int DIR_SW= 4;
	public static final int DIR_SE= 5;

	public static final int WIN_POINTS= 1;
	public static final int DRAW_POINTS= 0;
	public static final int LOSE_POINTS= -1;

	public static final int MAX_TURNS= 200;
	
	// shared vars
	public int acceptedClients= 0;

	private HexFrame hf;
	private Color boardColor, gutterColor;
	private int port;
	private String yellowName, blackName;
	private ClientHandler yellowClient, blackClient;
	private int currentPlayer= COLOR_YELLOW;
	private boolean gameEnded= false;
	private int turnnum= 1;
	private boolean invalidMove= false;
	private Timer timer;
	private TurnTimeoutTask timeoutTask= null;
	private int winner= -1;
	private int humanPlayers= 0;
	private int accepted= 0;
	private String[] humanPlayerNames;
	private boolean gui, log, timeout;
	private HexagonalBoard board; // used when no gui
	private PrintStream logstream;

	// 5 seconds per turn (5000 milliseconds)
	private long turntime= (long)(5000);

	public Main(int _port, int _humanPlayers, boolean _gui, boolean _log)
	{
		port= _port;
		gui= _gui;
		log= _log;
		humanPlayers= _humanPlayers;
		timer= new Timer();
		timeout= false;

		humanPlayerNames= new String[2];
		humanPlayerNames[0]= "HumanPlayer1";
		humanPlayerNames[1]= "HumanPlayer2";

		boardColor= new Color((float)0.6, (float)1.0, (float)0.6);
		gutterColor= new Color((float)0.2, (float)0.6, (float)0.2);

		if (gui)
		{
			setupFrame();
		}
		else
		{
			board= new HexagonalBoard(7);
		}

		initBoard();

		if (gui)
			hf.setVisible(true);
	}

	private void setupFrame()
	{
		hf= new HexFrame("Knockabout", 800, 600, HexFrame.TYPE_HEXAGONAL, 7, Color.black);
		hf.showGrid();
		hf.getHexPanel().setScale(true);
		colorBoard();
		colorGutter();
	}
		
	private void colorBoard()
	{
		Iterator i= hf.getHexPanel().getBoard().tilesIterator();

		while (i.hasNext())
		{
			Hexagon h= (Hexagon)i.next();
			hf.getHexPanel().setHexFillColor(h.getHexX(), h.getHexY(), boardColor);
		}
	}

	private void colorGutter()
	{
		hf.getHexPanel().setHexFillColor(0, 0, gutterColor);
		hf.getHexPanel().setHexFillColor(1, 0, gutterColor);
		hf.getHexPanel().setHexFillColor(2, 0, gutterColor);
		hf.getHexPanel().setHexFillColor(3, 0, gutterColor);
		hf.getHexPanel().setHexFillColor(4, 0, gutterColor);
		hf.getHexPanel().setHexFillColor(5, 0, gutterColor);
		hf.getHexPanel().setHexFillColor(6, 0, gutterColor);
		hf.getHexPanel().setHexFillColor(7, 1, gutterColor);
		hf.getHexPanel().setHexFillColor(8, 2, gutterColor);
		hf.getHexPanel().setHexFillColor(9, 3, gutterColor);
		hf.getHexPanel().setHexFillColor(10, 4, gutterColor);
		hf.getHexPanel().setHexFillColor(11, 5, gutterColor);
		hf.getHexPanel().setHexFillColor(12, 6, gutterColor);
		hf.getHexPanel().setHexFillColor(12, 7, gutterColor);
		hf.getHexPanel().setHexFillColor(12, 8, gutterColor);
		hf.getHexPanel().setHexFillColor(12, 9, gutterColor);
		hf.getHexPanel().setHexFillColor(12, 10, gutterColor);
		hf.getHexPanel().setHexFillColor(12, 11, gutterColor);
		hf.getHexPanel().setHexFillColor(12, 12, gutterColor);
		hf.getHexPanel().setHexFillColor(11, 12, gutterColor);
		hf.getHexPanel().setHexFillColor(10, 12, gutterColor);
		hf.getHexPanel().setHexFillColor(9, 12, gutterColor);
		hf.getHexPanel().setHexFillColor(8, 12, gutterColor);
		hf.getHexPanel().setHexFillColor(7, 12, gutterColor);
		hf.getHexPanel().setHexFillColor(6, 12, gutterColor);
		hf.getHexPanel().setHexFillColor(5, 11, gutterColor);
		hf.getHexPanel().setHexFillColor(4, 10, gutterColor);
		hf.getHexPanel().setHexFillColor(3, 9, gutterColor);
		hf.getHexPanel().setHexFillColor(2, 8, gutterColor);
		hf.getHexPanel().setHexFillColor(1, 7, gutterColor);
		hf.getHexPanel().setHexFillColor(0, 6, gutterColor);
		hf.getHexPanel().setHexFillColor(0, 5, gutterColor);
		hf.getHexPanel().setHexFillColor(0, 4, gutterColor);
		hf.getHexPanel().setHexFillColor(0, 3, gutterColor);
		hf.getHexPanel().setHexFillColor(0, 2, gutterColor);
		hf.getHexPanel().setHexFillColor(0, 1, gutterColor);
	}

	private boolean isGutter(int x, int y)
	{
		if (   (x == 0 && y == 0) || (x == 1 && y == 0) || (x == 2 && y == 0) || (x == 3 && y == 0)
		    || (x == 4 && y == 0) || (x == 5 && y == 0) || (x == 6 && y == 0) || (x == 7 && y == 1)
			 || (x == 8 && y == 2) || (x == 9 && y == 3) || (x == 10 && y == 4) || (x == 11 && y == 5)
			 || (x == 12 && y == 6) || (x == 12 && y == 7) || (x == 12 && y == 8) || (x == 12 && y == 9)
			 || (x == 12 && y == 10) || (x == 12 && y == 11) || (x == 12 && y == 12) || (x == 11 && y == 12)
			 || (x == 10 && y == 12) || (x == 9 && y == 12) || (x == 8 && y == 12) || (x == 7 && y == 12)
			 || (x == 6 && y == 12) || (x == 5 && y == 11) || (x == 4 && y == 10) || (x == 3 && y == 9)
			 || (x == 2 && y == 8) || (x == 1 && y == 7) || (x == 0 && y == 6) || (x == 0 && y == 5)
			 || (x == 0 && y == 4) || (x == 0 && y == 3) || (x == 0 && y == 2) || (x == 0 && y == 1) )
			return true;
		else
			return false;
	}

	public void setInvalidMove()
	{
		invalidMove= true;
	}

	public void setTimeout()
	{
		timeout= true;
	}

	public void setHumanPlayerName(int num, String _name)
	{
		humanPlayerNames[num-1]= _name;
	}

	public static void main(String[] args)
	{
		boolean gui= true;
		boolean log= true;
		int humanPlayers= 0;
		String hpn1= "HumanPlayer1", hpn2= "HumanPlayer2";

		if (args == null || args.length < 1)
			usage();


		for (int i= 0; i < args.length; i++)
		{
			if (args[i].equals("-ng"))
				gui= false;
			else if (args[i].equals("-nl"))
				log= false;
			else if (args[i].equals("-h"))
				humanPlayers= Integer.parseInt(args[++i]);
			else if (args[i].equals("-hpn1"))
				hpn1= args[++i];
			else if (args[i].equals("-hpn2"))
				hpn2= args[++i];
		}

		Main m= new Main(Integer.parseInt(args[0]), humanPlayers, gui, log);

		m.setHumanPlayerName(1, hpn1);
		m.setHumanPlayerName(2, hpn2);

		m.start();
   }

	public static void usage()
	{
		System.out.println("Usage: Main <port> [op1] [op2]  ... ");
		System.out.println("   Options:");
		System.out.println("      -ng             means no gui");
		System.out.println("      -nl             means no log");
		System.out.println("      -h #            set the number of human players to # (default 0). ");
		System.out.println("      -hpn1 <name>    set the humanplayer1 name to <name>");
		System.out.println("      -hpn2 <name>    set the humanplayer2 name to <name>");
		System.exit(-1);
	}

	public int currentPlayer()
	{
		return currentPlayer;
	}

	public void run()
	{
		try
		{
			System.out.println("Starting server...");
			ServerSocket ss= new ServerSocket(port);

			if (humanPlayers > 0)
			{
				System.out.println("Spawning human player clients...");
				System.out.println("Note: each human player must click on the board to request a start.");
				System.out.println("Note #2: server-side timers are disabled when there are human players.");

				for (int i= 0; i < humanPlayers; i++)
				{
					if (!gui)
					{
						System.out.println("Can't have human players without a GUI....");
						System.exit(-1);
					}

					HumanPlayer hp= new HumanPlayer(this, hf.getHexPanel(), humanPlayerNames[i], port);
					hp.start();
				}
			}

			System.out.println("Started. Waiting for incoming connections on port " + port + " ...");
			
			while (accepted < 2) // 2-player game
			{
				Socket client= ss.accept();

				if (accepted == 0)
				{
					yellowClient= new ClientHandler(client, this, Main.COLOR_YELLOW);
					yellowClient.start();
				}
				else if (accepted == 1)
				{
					blackClient= new ClientHandler(client, this, Main.COLOR_BLACK);
					blackClient.start();
				}

				accepted++;
			}

			ss.close();
		}
		catch(Exception e)
		{
			e.printStackTrace();
			System.exit(-1);
		}
	}

	public void openLog()
	{
		if (log)
		{
			Calendar cal= Calendar.getInstance();
			String datestring= "" + cal.get(Calendar.DAY_OF_MONTH) +
			                   cal.get(Calendar.MONTH) +
		      	             cal.get(Calendar.YEAR) + "_" +
		         	          cal.get(Calendar.HOUR_OF_DAY) +
		            	       cal.get(Calendar.MINUTE);

			String logname= replaceSpaces(getPlayerName(Main.COLOR_YELLOW)) + "_" +
		   	             replaceSpaces(getPlayerName(Main.COLOR_BLACK)) + "_" +
		      	          datestring + ".log";

			String dirSep= System.getProperty("file.separator");

			try
			{
				logstream= new PrintStream(new FileOutputStream("log" + dirSep + logname));
			}
			catch(Exception e)
			{
				e.printStackTrace();
				System.exit(-1);
			}
		}
	}

	public void log(String msg)
	{
		if (log)
			logstream.println(msg);
	}

	public static void fatalError(String s)
	{
		System.out.println("Error: " + s);
		System.exit(-1);
	}

	public int convertDir(String dir)
	{
		if      (dir.equals("NW")) return Main.DIR_NW;
		else if (dir.equals("NE")) return Main.DIR_NE;
		else if (dir.equals("E")) return Main.DIR_E;
		else if (dir.equals("W")) return Main.DIR_W;
		else if (dir.equals("SW")) return Main.DIR_SW;
		else if (dir.equals("SE")) return Main.DIR_SE;
		else return -1;
	}

	public String convertDir(int dir)
	{
		if      (dir == Main.DIR_NW) return "NW";
		else if (dir == Main.DIR_NE) return "NE";
		else if (dir == Main.DIR_E) return "E";
		else if (dir == Main.DIR_W) return "W";
		else if (dir == Main.DIR_SW) return "SW";
		else if (dir == Main.DIR_SE) return "SE";
		else return null;
	}

	public String convertColor(int color)
	{
		if (color == Main.COLOR_YELLOW) return "YELLOW";
		else if (color == Main.COLOR_BLACK) return "BLACK";
		return null;
	}

	public int convertColor(String color)
	{
		if (color.equals("YELLOW")) return Main.COLOR_YELLOW;
		else if (color.equals("BLACK")) return Main.COLOR_BLACK;
		return -1;
	}

	private String replaceSpaces(String s)
	{
		int spaceindex= s.indexOf(" ");

		if (spaceindex < 0)
			return s;

		String trailing= "";
		if (spaceindex != s.length()-1)
			trailing= s.substring(spaceindex+1);

		return (replaceSpaces(s.substring(0, spaceindex) + "_" + trailing));
	}

	private void initBoard()
	{
		Die yd4_1= new Die(Main.COLOR_YELLOW, 4, 1);
		Die yd4_2= new Die(Main.COLOR_YELLOW, 4, 1);
		Die yd4_3= new Die(Main.COLOR_YELLOW, 4, 1);
		Die yd4_4= new Die(Main.COLOR_YELLOW, 4, 1);
		Die yd6_1= new Die(Main.COLOR_YELLOW, 6, 2);
		Die yd6_2= new Die(Main.COLOR_YELLOW, 6, 2);
		Die yd6_3= new Die(Main.COLOR_YELLOW, 6, 2);
		Die yd8_1= new Die(Main.COLOR_YELLOW, 8, 3);
		Die yd8_2= new Die(Main.COLOR_YELLOW, 8, 3);
		Die bd4_1= new Die(Main.COLOR_BLACK, 4, 1);
		Die bd4_2= new Die(Main.COLOR_BLACK, 4, 1);
		Die bd4_3= new Die(Main.COLOR_BLACK, 4, 1);
		Die bd4_4= new Die(Main.COLOR_BLACK, 4, 1);
		Die bd6_1= new Die(Main.COLOR_BLACK, 6, 2);
		Die bd6_2= new Die(Main.COLOR_BLACK, 6, 2);
		Die bd6_3= new Die(Main.COLOR_BLACK, 6, 2);
		Die bd8_1= new Die(Main.COLOR_BLACK, 8, 3);
		Die bd8_2= new Die(Main.COLOR_BLACK, 8, 3);

		if (gui)
		{
			yd4_1.resetIcon();
			yd4_2.resetIcon();
			yd4_3.resetIcon();
			yd4_4.resetIcon();
			yd6_1.resetIcon();
			yd6_2.resetIcon();
			yd6_3.resetIcon();
			yd8_1.resetIcon();
			yd8_2.resetIcon();
			bd4_1.resetIcon();
			bd4_2.resetIcon();
			bd4_3.resetIcon();
			bd4_4.resetIcon();
			bd6_1.resetIcon();
			bd6_2.resetIcon();
			bd6_3.resetIcon();
			bd8_1.resetIcon();
			bd8_2.resetIcon();
		}


		getBoard().putObject(3, 4, bd4_1);
		getBoard().putObject(3, 5, yd4_1);
		getBoard().putObject(4, 3, bd6_1);
		getBoard().putObject(4, 7, yd6_1);
		getBoard().putObject(5, 2, bd8_1);
		getBoard().putObject(5, 5, bd4_2);
		getBoard().putObject(5, 6, yd4_2);
		getBoard().putObject(5, 9, yd8_1);
		getBoard().putObject(6, 4, bd6_2);
		getBoard().putObject(6, 8, yd6_2);
		getBoard().putObject(7, 3, bd8_2);
		getBoard().putObject(7, 6, bd4_3);
		getBoard().putObject(7, 7, yd4_3);
		getBoard().putObject(7, 10, yd8_2);
		getBoard().putObject(8, 5, bd6_3);
		getBoard().putObject(8, 9, yd6_3);
		getBoard().putObject(9, 7, bd4_4);
		getBoard().putObject(9, 8, yd4_4);

	}

	private int nextX(int x, int dir)
	{
		switch(dir)
		{
			case Main.DIR_NW: return (x-1);
			case Main.DIR_NE: return (x-1);
			case Main.DIR_W: return (x);
			case Main.DIR_E: return (x);
			case Main.DIR_SW: return (x+1);
			case Main.DIR_SE: return (x+1);
			default: return -10000;
		}
	}

	private int nextY(int y, int dir)
	{
		switch(dir)
		{
			case Main.DIR_NW: return (y-1);
			case Main.DIR_NE: return (y);
			case Main.DIR_W: return (y-1);
			case Main.DIR_E: return (y+1);
			case Main.DIR_SW: return (y);
			case Main.DIR_SE: return (y+1);
			default: return -10000;
		}
	}

	private class XYDir
	{
		public int x, y, dir;

		public XYDir() { }
	}


	private XYDir gutterNextSet(int x, int y, int dir)
	{
		XYDir set= new XYDir();
		set.x= nextX(x, dir);
		set.y= nextY(y, dir);
		set.dir= dir;

		if      (x == 1 && y == 0 && dir == Main.DIR_NE)
			set.dir= Main.DIR_E;
		else if (x == 0 && y == 1 && dir == Main.DIR_W)
			set.dir= Main.DIR_SW;
		else if (x == 5 && y == 0 && dir == Main.DIR_SW)
			set.dir= Main.DIR_SE;
		else if (x == 7 && y == 1 && dir == Main.DIR_NW)
			set.dir= Main.DIR_NE;
		else if (x == 11 && y == 5 && dir == Main.DIR_SE)
			set.dir= Main.DIR_E;
		else if (x == 12 && y == 7 && dir == Main.DIR_W)
			set.dir= Main.DIR_NW;
		else if (x == 12 && y == 11 && dir == Main.DIR_E)
			set.dir= Main.DIR_NE;
		else if (x == 11 && y == 12 && dir == Main.DIR_SW)
			set.dir= Main.DIR_W;
		else if (x == 7 && y == 12 && dir == Main.DIR_NE)
			set.dir= Main.DIR_NW;
		else if (x == 5 && y == 11 && dir == Main.DIR_SE)
			set.dir= Main.DIR_SW;
		else if (x == 1 && y == 7 && dir == Main.DIR_NW)
			set.dir= Main.DIR_W;
		else if (x == 0 && y == 5 && dir == Main.DIR_E)
			set.dir= Main.DIR_SE;

		return set;
	}

	// ---------------------------------
	// API to be used by client handlers
	
	public HexagonalBoard getBoard()
	{
		if (gui)
		{
			return (HexagonalBoard)hf.getHexPanel().getBoard();
		}
		else
		{
			return board;
		}
	}

	public void setPlayerName(int color, String name)
	{
		if (color == Main.COLOR_YELLOW)
			yellowName= name;
		else if (color == Main.COLOR_BLACK)
			blackName= name;
	}

	public String getPlayerName(int color)
	{
		if (color == Main.COLOR_YELLOW) return yellowName;
		else if (color == Main.COLOR_BLACK) return blackName;
		return null;
	}

	public int getWinner()
	{
		return winner;
	}

	public void broadcastMsg(String msg)
	{
		yellowClient.send(msg);
		blackClient.send(msg);
	}

	public void sendtoYellow(String msg)
	{
		yellowClient.send(msg);
	}

	public void sendtoBlack(String msg)
	{
		blackClient.send(msg);
	}

	public synchronized void resetTimer()
	{
		if (humanPlayers == 0) // disable timers when there are human players
		{                      // note: this is only server side. you can still
			                    // enable the client-side timers when playing
			                    // human players play against AI players
			if (timeoutTask != null)
				timeoutTask.cancel();
	
			if (!gameEnded)
			{
				timeoutTask= new TurnTimeoutTask(this, turnnum);
				timer.schedule(timeoutTask, turntime);
			}
		}
	}

	public int getTurnNum()
	{
		return turnnum;
	}

	public void nextTurn(String broadcastMsg, int thisturnNum)
	{
		if (thisturnNum == turnnum)
		{
			if (currentPlayer == COLOR_YELLOW)
				currentPlayer= COLOR_BLACK;
			else
				currentPlayer= COLOR_YELLOW;

			turnnum++;
			resetTimer();
			broadcastMsg(broadcastMsg);
			System.out.println("Turn " + turnnum);
			log("Turn " + (turnnum-1) + ": " + broadcastMsg);
		}
		// else, some outer process changed the turn so don't change it
	}

	public boolean isValidGutterMove(int x, int y, int dir)
	{
		if (x == 0 && y == 0)
			if (dir == Main.DIR_SW || dir == Main.DIR_E)
				return true;

		if (x >= 1 && x <= 5 && y == 0)
			if (dir == Main.DIR_NE || dir == Main.DIR_SW)
				return true;

		if (x == 6 && y == 0)
			if (dir == Main.DIR_NE || dir == Main.DIR_SE)
				return true;

		if (   (x == 7 && y == 1) || (x == 8 && y == 2) || (x == 9 && y == 3)
		    || (x == 10 && y == 4) || (x == 11 && y == 5))
			 if (dir == Main.DIR_NW || dir == Main.DIR_SE)
			 	return true;

		if (x == 12 && y == 6)
			if (dir == Main.DIR_NW || dir == Main.DIR_E)
				return true;

		if (x == 12 && y >= 7 && y <= 11)
			if (dir == Main.DIR_W || dir == Main.DIR_E)
				return true;

		if (x == 12 && y == 12)
			if (dir == Main.DIR_W || dir == Main.DIR_NE)
				return true;

		if (x <= 11 && x >= 7 && y == 12)
			if (dir == Main.DIR_SW || dir == Main.DIR_NE)
				return true;

		if (x == 6 && y == 12)
			if (dir == Main.DIR_SW || dir == Main.DIR_NW)
				return true;

		if (   (x == 5 && y == 11) || (x == 4 && y == 10) || (x == 3 && y == 9)
		    || (x == 2 && y == 8) || (x == 1 && y == 7))
			 if (dir == Main.DIR_NW || dir == Main.DIR_SE)
			 	return true;

		if (x == 0 && y == 6)
			if (dir == Main.DIR_W || dir == Main.DIR_SE)
				return true;

		if (x == 0 && y <= 5 && y >= 1)
			if (dir == Main.DIR_W || dir == Main.DIR_E)
				return true;

		return false;
	}

	// returns -1 on faliure, 0 on success but no collision, > 0 if success
	// and collision (number returns is the last reroll)
	public int applyMove(int x, int y, int dir, int playerColor)
	{
		HexagonalBoard hexboard= getBoard();
		boolean gutterMove= false;

		// check out of bounds
		if (!hexboard.isValidCoord(x,y))
			return -1;

		// make sure a die is in that position, and make sure it belongs to the
		// current player
		if (!(   hexboard.getHexagon(x,y).isOccupied()
		      && ((Die)hexboard.getHexagon(x,y).getObj()).getColor() == playerColor))
			return -1;

		if (isGutter(x,y))
		{
			if (!isValidGutterMove(x, y, dir))
				return -1;

			gutterMove= true;
		}

		// we've done all the validation we need, now apply the move
		int movementPoints= ((Die)hexboard.getHexagon(x,y).getObj()).curVal();
		int curX= x;
		int curY= y;
		int nextX= x;
		int nextY= y;
		int retval= 0;
		boolean collision= false;

		int curDie_x= x;
		int curDie_y= y;
		Die curDie= ((Die)hexboard.getHexagon(x,y).getObj());

		while(movementPoints > 0)
		{
			if (!gutterMove)
			{
				nextX= nextX(curX, dir);
				nextY= nextY(curY, dir);
			}
			else
			{
				XYDir set= gutterNextSet(curX, curY, dir);
				nextX= set.x;
				nextY= set.y;
				dir= set.dir;
			}

			if (!hexboard.isValidCoord(nextX,nextY))
				break;	// Reached the end of the screen

			if (hexboard.getHexagon(nextX, nextY).isOccupied())
			{
				collision= true;
				hexboard.removeObject(curDie_x, curDie_y);
				hexboard.putObject(curX, curY, curDie);
				curDie_x= nextX;
				curDie_y= nextY;
				curDie= ((Die)hexboard.getHexagon(nextX, nextY).getObj());
			}
			else
			{
				movementPoints--;
			}

			curX= nextX;
			curY= nextY;
		}

		hexboard.removeObject(curDie_x, curDie_y);
		hexboard.putObject(curX, curY, curDie);

		if (collision)
		{
			curDie.reroll();
			if (gui) curDie.resetIcon();
			retval= curDie.curVal();
		}

		if (gui)
			hf.repaint();

		return retval;
	}

	public boolean gameEnded()
	{
		return gameEnded;
	}

	private void updateHighScore()
	{
		HighScore highscore= HighScore.loadHighScore("highscore");
		System.out.println("Updating highscore files...");

		if (winner == Main.COLOR_BLACK)
		{
			highscore.update(getPlayerName(Main.COLOR_BLACK), Main.WIN_POINTS);
			highscore.update(getPlayerName(Main.COLOR_YELLOW), Main.LOSE_POINTS);
		}
		else if (winner == Main.COLOR_YELLOW)
		{
			highscore.update(getPlayerName(Main.COLOR_YELLOW), Main.WIN_POINTS);
			highscore.update(getPlayerName(Main.COLOR_BLACK), Main.LOSE_POINTS);
		}
		else
		{
			highscore.update(getPlayerName(Main.COLOR_YELLOW), Main.DRAW_POINTS);
			highscore.update(getPlayerName(Main.COLOR_BLACK), Main.DRAW_POINTS);
		}

		HighScore.updateHighScoreFile(highscore);
	}

	public boolean checkGameEnd(String[] move)
	{
		int blackAlive= 0;
		int yellowAlive= 0;

		Iterator i= getBoard().tilesIterator();

		if (invalidMove)
		{
			gameEnded= true;

			System.out.println("!!! INVALID MOVE RECEIVED !!!");

			if (move[0].equals("BLACK"))
				winner= Main.COLOR_YELLOW;
			else if (move[0].equals("YELLOW"))
				winner= Main.COLOR_BLACK;

			String gameend_msg= "GAME ENDED; " + move[0] + " " + move[1] + " " + move[2]
				+ " " + move[3] + "; WINNER IS " + convertColor(winner);
			broadcastMsg(gameend_msg);
			log(gameend_msg);
			updateHighScore();
			return true;
		}
		else if (timeout)
		{
			gameEnded= true;

			System.out.println("!!! TIMEOUT !!!");

			if (move[0].equals("BLACK"))
				winner= Main.COLOR_YELLOW;
			else if (move[0].equals("YELLOW"))
				winner= Main.COLOR_BLACK;

			String gameend_msg= "GAME ENDED; " + move[0] + " " + move[1] + "; WINNER IS " + convertColor(winner);
			broadcastMsg(gameend_msg);
			log(gameend_msg);
			updateHighScore();
			return true;
		}
		else if (turnnum >= Main.MAX_TURNS)
		{
			gameEnded= true;

			System.out.println("!!! DRAW !!!");

			winner= -1;

			String gameend_msg= "GAME ENDED; " + move[0] + " " + move[1] + " " + move[2]
				+ " " + move[3] + "; DRAW";
			broadcastMsg(gameend_msg);
			log(gameend_msg);
			updateHighScore();
			return true;
		}

		while (i.hasNext())
		{
			Hexagon h= (Hexagon)i.next();

			if (h.isOccupied())
			{
				int color= ((Die)h.getObj()).getColor();

				if (!isGutter(h.getHexX(), h.getHexY()))
				{
					if      (color == Main.COLOR_YELLOW) yellowAlive++;
					else if (color == Main.COLOR_BLACK) blackAlive++;
				}
			}

			if (blackAlive > 4 && yellowAlive > 4)
				return false;
		}

		if (blackAlive > 4 && yellowAlive <= 4)
		{
			gameEnded= true;
			String gameend_msg= "GAME ENDED; " + move[0] + " " + move[1] + " " + move[2]
				+ " " + move[3] + "; WINNER IS BLACK";
			winner= Main.COLOR_BLACK;
			broadcastMsg(gameend_msg);
			log(gameend_msg);
			updateHighScore();
			return true;
		}
		else if (yellowAlive > 4  && blackAlive <= 4)
		{
			gameEnded= true;
			winner= Main.COLOR_YELLOW;
			String gameend_msg= "GAME ENDED; " + move[0] + " " + move[1] + " " + move[2]
				+ " " + move[3] + "; WINNER IS YELLOW";
			broadcastMsg(gameend_msg);
			log(gameend_msg);
			updateHighScore();
			return true;
		}

		return false;
	}

	public synchronized void processMessage(String msg, int color)
	{
		if (msg == null)
			return;

		int turnnum= getTurnNum();

		System.out.println("Processing Message: " + msg);

		if (msg.startsWith("START "))
		{
			// startgame messages

			String name= msg.substring(6, msg.length());
			setPlayerName(color, name);
			System.out.println("# Got start message from " + convertColor(color) +
				". Player name: " + name);

			acceptedClients++;
			if (acceptedClients == 2)
			{
				String startgame_msg= "GAME STARTED; YOU ARE ";
				resetTimer();
				openLog();
				sendtoBlack(startgame_msg + "BLACK. OPPONENT IS " + getPlayerName(Main.COLOR_YELLOW));
				sendtoYellow(startgame_msg + "YELLOW. OPPONENT IS " + getPlayerName(Main.COLOR_BLACK));
			}
		}
		else if (msg.startsWith("BLACK ") || msg.startsWith("YELLOW "))
		{
			// movement messages

			String[] words= msg.split(" ");

			if (words.length != 4)
			{
				//nextTurn(words[0] + " INVALID MOVE", turnnum);
				setInvalidMove();
				checkGameEnd(words);
				return;
			}

			int x_coord, y_coord, dir;

			try
			{
				x_coord= Integer.parseInt(words[1]);
				y_coord= Integer.parseInt(words[2]);
				dir= convertDir(words[3]);
			}
			catch (Exception e)
			{
				//nextTurn(words[0] + " INVALID MOVE", turnnum);
				setInvalidMove();
				checkGameEnd(words);
				return;
			}

			if (   words[0].equals("BLACK")
			    && (   currentPlayer() != Main.COLOR_BLACK
				     || color != Main.COLOR_BLACK))
			{
				// cheaters or late message
				System.out.println("!!! Warning: Message received out of turn");
				return;
			}

			if (   words[0].equals("YELLOW")
			    && (   currentPlayer() != Main.COLOR_YELLOW
				     || color != Main.COLOR_YELLOW))
			{
				// cheaters or late message
				System.out.println("!!! Warning: Message received out of turn");
				return;
			}

			int retval= applyMove(x_coord, y_coord, dir, color);

			if (retval < 0)
			{
				//nextTurn(words[0] + " INVALID MOVE", turnnum);
				setInvalidMove();
				checkGameEnd(words);
				return;
			}
			else if (retval == 0)
			{
				if (!checkGameEnd(words))
				{
					nextTurn(words[0] + " " + x_coord + " " + y_coord + " " + words[3], turnnum);
				}

				return;
			}
			else if (retval > 0)
			{
				if (!checkGameEnd(words))
				{
					nextTurn(words[0] + " " + x_coord + " " + y_coord + " " + words[3]
						+ " " + retval, turnnum);
				}

				return;
			}

			return;
		}

	}

}


class TurnTimeoutTask extends TimerTask
{
	private Main mainref;
	private int nextturnNum;

	public TurnTimeoutTask(Main _mainref, int _nextturnNum)
	{
		mainref= _mainref;
		nextturnNum= _nextturnNum;
	}

	public void run()
	{
		if (!mainref.gameEnded())
		{
			mainref.setTimeout();
			String[] move= new String[2];
			move[0]= mainref.convertColor(mainref.currentPlayer());
			move[1]= "TIMEOUT";

			mainref.checkGameEnd(move);
			System.exit(-1);
		}
	}
}




